@@ -1,23 +1,31 @@
-import surpyval
 import numpy as np
-from surpyval import nonparametric as nonp
+from surpyval.utils import xcnt_to_xrd
 from surpyval.nonparametric.nonparametric_fitter import NonParametricFitter
 
-def nelson_aalen(x, c=None, n=None, **kwargs):
-    x, r, d = surpyval.xcnt_to_xrd(x, c, n, **kwargs)
 
-    h = d/r
-    H = np.cumsum(h)
+def na(r, d):
+    H = np.cumsum(d / r)
+    H[np.isnan(H)] = np.inf
     R = np.exp(-H)
-    return x, r, d, R
+    return R
+
+
+def nelson_aalen(x, c, n, t):
+    xrd = xcnt_to_xrd(x, c, n, t)
+    out = {k: v for k, v in zip(['x', 'r', 'd'], xrd)}
+    out['R'] = na(out['r'], out['d'])
+    return out
+
 
 class NelsonAalen_(NonParametricFitter):
     r"""
-    Nelson-Aalen estimator class. Returns a `NonParametric` object from method :code:`fit()` Calculates the Non-Parametric estimate of the survival function using:
+    Nelson-Aalen estimator class. Returns a `NonParametric`
+    object from method :code:`fit()` Calculates the Non-Parametric
+    estimate of the survival function using:
 
     .. math::
         R(x) = e^{-\sum_{i:x_{i} \leq x}^{} \frac{d_{i} }{r_{i}}}
-    
+
     Examples
     --------
     >>> import numpy as np
@@ -30,4 +38,5 @@ class NelsonAalen_(NonParametricFitter):
     def __init__(self):
         self.how = 'Nelson-Aalen'
 
-NelsonAalen = NelsonAalen_()
\ No newline at end of file
+
+NelsonAalen = NelsonAalen_()
